@@ -24,9 +24,11 @@ git reset --hard
 git pull
 git clean -dfx
 git reset --hard
-./build-macosx || exit 1
+./build-macos || exit 1
 make dosbox-x.app || exit 1
 cp CHANGELOG CHANGELOG.txt || exit 1
+cp COPYING COPYING.txt || exit 1
+cp contrib/macos/readme.txt README.txt || exit 1
 
 cd "$top/dosbox-x-sdl2" || exit 1
 git clean -dfx
@@ -37,14 +39,16 @@ git reset --hard
 git pull
 git clean -dfx
 git reset --hard
-./build-macosx-sdl2 || exit 1
+./build-macos-sdl2 || exit 1
 make dosbox-x.app || exit 1
 cp CHANGELOG CHANGELOG.txt || exit 1
+cp COPYING COPYING.txt || exit 1
+cp contrib/macos/readme.txt README.txt || exit 1
 
 cd "$top" || exit 1
 echo "Packing up now..."
 
-zip -r -9 "$name" dosbox-x/dosbox-x.app dosbox-x/CHANGELOG.txt dosbox-x-sdl2/dosbox-x.app dosbox-x-sdl2/CHANGELOG.txt || exit 1
+zip -r -9 "$name" dosbox-x/dosbox-x.app dosbox-x/CHANGELOG.txt dosbox-x/COPYING.txt dosbox-x/README.txt dosbox-x-sdl2/dosbox-x.app dosbox-x-sdl2/CHANGELOG.txt dosbox-x-sdl2/COPYING.txt dosbox-x-sdl2/README.txt || exit 1
 
 exit 0
 
