@@ -42,7 +42,7 @@ top=`pwd`
 cd "$top" || exit 1
 
 hxdosdir="dosbox-x-mingw-hx-dos"
-ziptool="$top/$hxdosdir/vs/tool/zip.exe"
+tooldir="$top/$hxdosdir/vs/tool"
 
 cd "$top/$hxdosdir" || exit 1
 git clean -dfx
@@ -57,25 +57,35 @@ git reset --hard
 
 # The paths of copied files and HX DOS Extender files 
 copydir="build-scripts/mingw/dosbox-x-mingw-hx-dos"
-hxdir="build-scripts/mingw/hxdos"
+hxdir="build-scripts/mingw/hxdos-bin"
 
 strip src/dosbox-x.exe || exit 1
 cp src/dosbox-x.exe dosbox-x.exe || exit 1
 $hxdir/pestub.exe -n dosbox-x.exe
+$tooldir/upx.exe --lzma -9 dosbox-x.exe
 cp CHANGELOG CHANGELOG.txt || exit 1
+cp COPYING COPYING.txt || exit 1
 cp dosbox-x.reference.conf dosbox-x.ref || exit 1
 cp dosbox-x.reference.full.conf dosbox-x.ref.full || exit 1
+cp contrib/windows/installer/inpout32.dll INPOUT32.DLL || exit 1
+cp contrib/fonts/FREECG98.BMP . || exit 1
+cp contrib/fonts/wqy_1?pt.bdf . || exit 1
+cp contrib/fonts/Nouveau_IBM.ttf . || exit 1
 cp $hxdir/DPMILD32.EXE . || exit 1
 cp $hxdir/HDPMI32.EXE . || exit 1
 cp $hxdir/HXGUIHLP.INI . || exit 1
 cp $hxdir/README.TXT . || exit 1
 cp $hxdir/WINSPOOL.DRV . || exit 1
 cp $hxdir/*.DLL . || exit 1
+mkdir -p drivez || exit 1
+cp contrib/windows/installer/drivez_readme.txt drivez/readme.txt || exit 1
+mkdir -p language || exit 1
+for i in `ls contrib/translations/` ; do cp contrib/translations/$i/*.lng language/; done
 
 cd "$top/$hxdosdir" || exit 1
 echo "Packing up now..."
 
-$ziptool -r -9 ../"$name" {CHANGELOG.txt,dosbox-x.exe,dosbox-x.ref,dosbox-x.ref.full,DPMILD32.EXE,HDPMI32.EXE,HXGUIHLP.INI,README.TXT,WINSPOOL.DRV,*.DLL} || exit 1
+$tooldir/zip.exe -r -9 ../"$name" {CHANGELOG.txt,COPYING.txt,dosbox-x.exe,dosbox-x.ref,dosbox-x.ref.full,FREECG98.BMP,wqy_1?pt.bdf,Nouveau_IBM.ttf,DPMILD32.EXE,HDPMI32.EXE,HXGUIHLP.INI,README.TXT,WINSPOOL.DRV,*.DLL,drivez/*,language/*} || exit 1
 cd ..
 
 exit 0
