@@ -241,9 +241,9 @@ bool Overlay_Drive::MakeDir(const char * dir) {
 				host_name = CodePageGuestToHost(pdir);
 				if (host_name!=NULL) {
 #if defined (WIN32)
-					temp=_wmkdir(host_name);
+					temp=_wmkdir_p(host_name);
 #else
-					temp=mkdir(host_name,0775);
+					temp=mkdir_p(host_name,0775);
 #endif
 					if (temp==0) madepdir=true;
 				}
@@ -254,9 +254,9 @@ bool Overlay_Drive::MakeDir(const char * dir) {
 	host_name = CodePageGuestToHost(newdir);
 	if (host_name!=NULL) {
 #if defined (WIN32)
-		temp=_wmkdir(host_name);
+		temp=_wmkdir_p(host_name);
 #else
-		temp=mkdir(host_name,0775);
+		temp=mkdir_p(host_name,0775);
 #endif
 	}
 	if (temp==0) {
@@ -1568,6 +1568,8 @@ void Overlay_Drive::remove_special_file_from_disk(const char* dosname, const cha
 }
 
 std::string Overlay_Drive::create_filename_of_special_operation(const char* dosname, const char* operation, bool expand) {
+	(void)expand;//unused
+
 	std::string res(dosname);
 	std::string::size_type s = std::string::npos; //CHECK DOS or host endings.... on update_cache
 	bool lead = false;
