@@ -32,13 +32,16 @@
 #include <map>
 #include <list>
 #include <string>
+#if defined(__MINGW32__) && !defined(HX_DOS)
+#include <imm.h>
+#endif
 using namespace std;
 
-extern bool dos_kernel_disabled, force_conversion, showdbcs, dbcs_sbcs;
+extern bool dos_kernel_disabled, force_conversion, showdbcs, dbcs_sbcs, enableime, tonoime;
 int msgcodepage = 0, FileDirExistUTF8(std::string &localname, const char *name);
 bool morelen = false, inmsg = false, loadlang = false, systemmessagebox(char const * aTitle, char const * aMessage, char const * aDialogType, char const * aIconType, int aDefaultButton);
 bool isSupportedCP(int newCP), CodePageHostToGuestUTF8(char *d/*CROSS_LEN*/,const char *s/*CROSS_LEN*/), CodePageGuestToHostUTF8(char *d/*CROSS_LEN*/,const char *s/*CROSS_LEN*/);
-void InitFontHandle(void), ShutFontHandle(void), menu_update_dynamic(void), menu_update_autocycle(void), update_bindbutton_text(void), set_eventbutton_text(const char *eventname, const char *buttonname), JFONT_Init();
+void InitFontHandle(void), ShutFontHandle(void), SetIME(void), menu_update_dynamic(void), menu_update_autocycle(void), update_bindbutton_text(void), set_eventbutton_text(const char *eventname, const char *buttonname), JFONT_Init();
 std::string langname = "", langnote = "", GetDOSBoxXPath(bool withexe=false);
 
 #define LINE_IN_MAXLEN 2048
@@ -200,8 +203,8 @@ void LoadMessageFile(const char * fname) {
 	if(*fname=='\0') return;//empty string=no languagefile
 
 	LOG(LOG_MISC,LOG_DEBUG)("Loading message file %s",fname);
-    std::string config_path, exepath=GetDOSBoxXPath();
-    Cross::GetPlatformConfigDir(config_path);
+    std::string config_path, res_path, exepath=GetDOSBoxXPath();
+    Cross::GetPlatformConfigDir(config_path), Cross::GetPlatformResDir(res_path);
 
 	FILE * mfile=fopen(fname,"rt");
 	if (!mfile) mfile=fopen((fname + std::string(".lng")).c_str(),"rt");
@@ -209,12 +212,24 @@ void LoadMessageFile(const char * fname) {
 	if (!mfile && exepath.size()) mfile=fopen((exepath + fname + ".lng").c_str(),"rt");
 	if (!mfile && config_path.size()) mfile=fopen((config_path + fname).c_str(),"rt");
 	if (!mfile && config_path.size()) mfile=fopen((config_path + fname + ".lng").c_str(),"rt");
+	if (!mfile && res_path.size()) mfile=fopen((res_path + fname).c_str(),"rt");
+	if (!mfile && res_path.size()) mfile=fopen((res_path + fname + ".lng").c_str(),"rt");
 	if (!mfile) mfile=fopen((std::string("languages/") + fname).c_str(),"rt");
 	if (!mfile) mfile=fopen((std::string("languages/") + fname + ".lng").c_str(),"rt");
 	if (!mfile && exepath.size()) mfile=fopen((exepath + "languages/" + fname).c_str(),"rt");
 	if (!mfile && exepath.size()) mfile=fopen((exepath + "languages/" + fname + ".lng").c_str(),"rt");
 	if (!mfile && config_path.size()) mfile=fopen((config_path + "languages/" + fname).c_str(),"rt");
 	if (!mfile && config_path.size()) mfile=fopen((config_path + "languages/" + fname + ".lng").c_str(),"rt");
+	if (!mfile && res_path.size()) mfile=fopen((res_path + "languages/" + fname).c_str(),"rt");
+	if (!mfile && res_path.size()) mfile=fopen((res_path + "languages/" + fname + ".lng").c_str(),"rt");
+	if (!mfile) mfile=fopen((std::string("language/") + fname).c_str(),"rt");
+	if (!mfile) mfile=fopen((std::string("language/") + fname + ".lng").c_str(),"rt");
+	if (!mfile && exepath.size()) mfile=fopen((exepath + "language/" + fname).c_str(),"rt");
+	if (!mfile && exepath.size()) mfile=fopen((exepath + "language/" + fname + ".lng").c_str(),"rt");
+	if (!mfile && config_path.size()) mfile=fopen((config_path + "language/" + fname).c_str(),"rt");
+	if (!mfile && config_path.size()) mfile=fopen((config_path + "language/" + fname + ".lng").c_str(),"rt");
+	if (!mfile && res_path.size()) mfile=fopen((res_path + "language/" + fname).c_str(),"rt");
+	if (!mfile && res_path.size()) mfile=fopen((res_path + "language/" + fname + ".lng").c_str(),"rt");
 #if defined(WIN32) && defined(C_SDL2)
     std::string localname = fname;
     if (!mfile && FileDirExistUTF8(localname, fname) == 1) mfile=fopen(localname.c_str(),"rt");
@@ -409,8 +424,21 @@ void MSG_Init() {
                 if (strlen(countrystr)>10) countrystr[0] = 0;
                 sprintf(cstr, "%s,%d", countrystr, msgcodepage);
                 SetVal("config", "country", cstr);
+                const char *imestr = section->Get_string("ime");
+                if (tonoime && !strcasecmp(imestr, "auto") && (msgcodepage == 932 || msgcodepage == 936 || msgcodepage == 949 || msgcodepage == 950 || msgcodepage == 951)) {
+                    tonoime = false;
+                    enableime = true;
+                    SetIME();
+                }
             }
         }
+        if (tonoime) {
+            tonoime = enableime = false;
+#if defined(WIN32) && !defined(HX_DOS)
+            ImmDisableIME((DWORD)(-1));
+#endif
+            SetIME();
+        }
 	}
 	else {
 		Prop_path* pathprop = section->Get_path("language");
