@@ -94,7 +94,7 @@ int switchoutput = -1;
 
 static unsigned long ttfSize = sizeof(DOSBoxTTFbi), ttfSizeb = 0, ttfSizei = 0, ttfSizebi = 0;
 static void * ttfFont = DOSBoxTTFbi, * ttfFontb = NULL, * ttfFonti = NULL, * ttfFontbi = NULL;
-extern int posx, posy, eurAscii, NonUserResizeCounter;
+extern int posx, posy, eurAscii, transparency, NonUserResizeCounter;
 extern bool rtl, gbk, chinasea, switchttf, force_conversion, blinking, showdbcs, loadlang, window_was_maximized;
 extern const char* RunningProgram;
 extern uint8_t ccount;
@@ -123,7 +123,7 @@ static bool blinkstate = false;
 bool colorChanged = false, justChanged = false, staycolors = false, firstsize = true, ttfswitch=false, switch_output_from_ttf=false;
 
 int menuwidth_atleast(int width), FileDirExistCP(const char *name), FileDirExistUTF8(std::string &localname, const char *name);
-void AdjustIMEFontSize(void), initcodepagefont(void), change_output(int output), MSG_Init(void), KEYBOARD_Clear(void), RENDER_Reset(void), DOSBox_SetSysMenu(void), GetMaxWidthHeight(unsigned int *pmaxWidth, unsigned int *pmaxHeight), resetFontSize(void), RENDER_CallBack( GFX_CallBackFunctions_t function );
+void AdjustIMEFontSize(void), initcodepagefont(void), change_output(int output), drawmenu(Bitu val), MSG_Init(void), KEYBOARD_Clear(void), RENDER_Reset(void), DOSBox_SetSysMenu(void), GetMaxWidthHeight(unsigned int *pmaxWidth, unsigned int *pmaxHeight), SetWindowTransparency(int trans), resetFontSize(void), RENDER_CallBack( GFX_CallBackFunctions_t function );
 bool isDBCSCP(void), InitCodePage(void), CodePageGuestToHostUTF16(uint16_t *d/*CROSS_LEN*/,const char *s/*CROSS_LEN*/), systemmessagebox(char const * aTitle, char const * aMessage, char const * aDialogType, char const * aIconType, int aDefaultButton);
 std::string GetDOSBoxXPath(bool withexe=false);
 
@@ -351,6 +351,22 @@ bool readTTF(const char *fName, bool bold, bool ital) {
             }
         }
     }
+    if (!ttf_fh) {
+        std::string res_path;
+        Cross::GetPlatformResDir(res_path);
+        struct stat info;
+        if (!stat(res_path.c_str(), &info) && (info.st_mode & S_IFDIR)) {
+            strcpy(ttfPath, res_path.c_str());
+            strcat(ttfPath, fName);
+            strcat(ttfPath, ".ttf");
+            ttf_fh = fopen(ttfPath, "rb");
+            if (!ttf_fh) {
+                strcpy(ttfPath, res_path.c_str());
+                strcat(ttfPath, fName);
+                ttf_fh = fopen(ttfPath, "rb");
+            }
+        }
+    }
     if (!ttf_fh) {
         std::string basedir = static_cast<Section_prop *>(control->GetSection("printer"))->Get_string("fontpath");
         if (basedir.back()!='\\' && basedir.back()!='/') basedir += CROSS_FILESPLIT;
@@ -697,10 +713,14 @@ void OUTPUT_TTF_Select(int fsize) {
         winPerc = ttf_section->Get_int("winperc");
         if (winPerc>100||(fsize==2&&GFX_IsFullscreen())||(fsize!=1&&fsize!=2&&(control->opt_fullscreen||static_cast<Section_prop *>(control->GetSection("sdl"))->Get_bool("fullscreen")))) winPerc=100;
         else if (winPerc<25) winPerc=25;
+#if defined(HX_DOS)
+        winPerc=100;
+#else
         if ((fsize==1||switchttf)&&winPerc==100) {
             winPerc=60;
             if (switchttf&&GFX_IsFullscreen()) GFX_SwitchFullScreen();
         }
+#endif
         fontSize = ttf_section->Get_int("ptsize");
         char512 = ttf_section->Get_bool("char512");
         showbold = ttf_section->Get_bool("bold");
@@ -1227,6 +1247,9 @@ void resetFontSize() {
 #endif
 		}
 		GFX_EndTextLines(true);
+#if defined(HX_DOS)
+		PIC_AddEvent(drawmenu, 200);
+#endif
 	}
 }
 
@@ -1335,6 +1358,10 @@ void ttf_reset() {
 #endif
 }
 
+void ttfreset(Bitu val) {
+    ttf_reset();
+}
+
 void ttf_setlines(int cols, int lins) {
     if (cols>0) SetVal("ttf", "cols", std::to_string(cols));
     if (lins>0) SetVal("ttf", "lins", std::to_string(lins));
@@ -1377,7 +1404,6 @@ void ttf_switch_on(bool ss=true) {
             OUTPUT_TTF_Select(3);
 #if DOSBOXMENU_TYPE == DOSBOXMENU_HMENU
             if (gl && GFX_IsFullscreen()) { // Hack for full-screen switch from OpenGL outputs
-                void GFX_SwitchFullScreen(void);
                 GFX_SwitchFullScreen();
                 GFX_SwitchFullScreen();
             }
@@ -1385,6 +1411,10 @@ void ttf_switch_on(bool ss=true) {
             resetreq = true;
         }
         resetFontSize();
+#ifdef C_SDL2
+        transparency = 0;
+        SetWindowTransparency(static_cast<Section_prop *>(control->GetSection("sdl"))->Get_int("transparency"));
+#endif
     }
 }
 
@@ -1431,6 +1461,10 @@ void ttf_switch_off(bool ss=true) {
         //if (GFX_IsFullscreen()) GFX_SwitchFullscreenNoReset();
         mainMenu.get_item("output_ttf").enable(false).refresh_item(mainMenu);
         RENDER_Reset();
+#ifdef C_SDL2
+        transparency = 0;
+        SetWindowTransparency(static_cast<Section_prop *>(control->GetSection("sdl"))->Get_int("transparency"));
+#endif
     }
 }
 #endif
