@@ -47,17 +47,16 @@
 # define MAX(a,b) std::max(a,b)
 #endif
 
-bool clearline=false, inshell=false;
+bool clearline=false, inshell=false, noassoc=false;
 int autofixwarn=3;
 extern int lfn_filefind_handle;
-extern bool ctrlbrk, gbk, rtl;
-extern bool DOS_BreakFlag;
-extern bool DOS_BreakConioFlag;
+extern bool ctrlbrk, gbk, rtl, dbcs_sbcs;
+extern bool DOS_BreakFlag, DOS_BreakConioFlag;
 extern uint16_t cmd_line_seg;
 #if defined(USE_TTF)
 extern bool ttf_dosv;
 #endif
-
+extern std::map<int, int> pc98boxdrawmap;
 void DOS_Shell::ShowPrompt(void) {
 	char dir[DOS_PATHLENGTH];
 	dir[0] = 0; //DOS_GetCurrentDir doesn't always return something. (if drive is messed up)
@@ -89,7 +88,7 @@ void DOS_Shell::ShowPrompt(void) {
 			case 'H': WriteOut("\b");   break;
 			case 'L': WriteOut("<"); break;
 			case 'N': WriteOut("%c",DOS_GetDefaultDrive()+'A'); break;
-			case 'P': WriteOut("%c:\\%s",DOS_GetDefaultDrive()+'A',dir); break;
+			case 'P': WriteOut("%c:\\",DOS_GetDefaultDrive()+'A'); WriteOut_NoParsing(dir, true); break;
 			case 'Q': WriteOut("="); break;
 			case 'S': WriteOut(" "); break;
 			case 'T': {
@@ -285,7 +284,7 @@ bool DOS_Shell::BuildCompletions(char * line, uint16_t str_len) {
                 if(att & DOS_ATTR_DIRECTORY) l_completion.emplace_back(qlname);
             } else {
                 const char *ext = strrchr(name, '.'); // file extension
-                if (ext && (strcmp(ext, ".BAT") == 0 || strcmp(ext, ".COM") == 0 || strcmp(ext, ".EXE") == 0))
+                if ((ext && (strcmp(ext, ".BAT") == 0 || strcmp(ext, ".COM") == 0 || strcmp(ext, ".EXE") == 0)) || hasAssociation(name).size())
                     // we add executables to a separate list and place that list in front of the normal files
                     executable.emplace_front(qlname);
                 else
@@ -324,6 +323,63 @@ static uint16_t GetWideCount(char *line, uint16_t str_index)
 	return count;
 }
 
+static uint16_t GetLastCount(char *line, uint16_t str_index)
+{
+	bool kanji_flag = false;
+	uint16_t count = 1;
+	for(uint16_t pos = 0 ; pos < str_index ; pos++) {
+		if(!kanji_flag) {
+			if(isKanji1(line[pos])) {
+				kanji_flag = true;
+			}
+			count = 1;
+		} else {
+			if(isKanji2(line[pos])) {
+				bool found=false;
+				if (IS_PC98_ARCH && line[pos-1] == 0xFFFFFF86) {
+                    for (auto it = pc98boxdrawmap.begin(); it != pc98boxdrawmap.end(); ++it)
+                        if (it->second ==line[pos]) {
+                            found=true;
+                            break;
+                        }
+                }
+				if (!found) count = 2;
+			}
+			kanji_flag = false;
+		}
+	}
+	return count;
+}
+
+static uint16_t GetRemoveCount(char *line, uint16_t str_index)
+{
+	bool kanji_flag = false;
+	uint16_t count = 1, total = 0;
+	for(uint16_t pos = 0 ; pos < str_index ; pos++) {
+		if(!kanji_flag) {
+			if(isKanji1(line[pos])) {
+				kanji_flag = true;
+			}
+			count = 1;
+		} else {
+			if(isKanji2(line[pos])) {
+				bool found=false;
+				if (IS_PC98_ARCH && line[pos-1] == 0xFFFFFF86) {
+                    for (auto it = pc98boxdrawmap.begin(); it != pc98boxdrawmap.end(); ++it)
+                        if (it->second ==line[pos]) {
+                            found=true;
+                            break;
+                        }
+                }
+				count = found?0:1;
+			}
+			kanji_flag = false;
+		}
+		total += count;
+	}
+	return total;
+}
+
 static void RemoveAllChar(char *line, uint16_t str_index)
 {
 	for ( ; str_index > 0 ; str_index--) {
@@ -341,12 +397,11 @@ static void RemoveAllChar(char *line, uint16_t str_index)
 
 static uint16_t DeleteBackspace(bool delete_flag, char *line, uint16_t &str_index, uint16_t &str_len)
 {
-	uint16_t count, pos;
-	uint16_t len;
+	uint16_t count, pos, len;
 
 	pos = str_index;
 	if(delete_flag) {
-		if(isKanji1(line[pos])) {
+		if(isKanji1(line[pos])&&line[pos+1]) {
 			pos += 2;
 		} else {
 			pos += 1;
@@ -362,7 +417,7 @@ static uint16_t DeleteBackspace(bool delete_flag, char *line, uint16_t &str_inde
 	}
 	if (delete_flag && str_index >= str_len)
 		return 0;
-	RemoveAllChar(line, str_len);
+	RemoveAllChar(line, GetRemoveCount(line, pos));
 	pos = delete_flag ? str_index : str_index - count;
 	while(pos < str_len - count) {
 		line[pos] = line[pos + count];
@@ -375,7 +430,7 @@ static uint16_t DeleteBackspace(bool delete_flag, char *line, uint16_t &str_inde
 	str_len -= count;
 	len = str_len;
 	DOS_WriteFile(STDOUT, (uint8_t *)line, &len);
-	pos = str_len;
+	pos = GetRemoveCount(line, str_len);
 	while(pos > str_index) {
 		backone();
 		if(CheckHat(line[pos - 1])) {
@@ -558,13 +613,19 @@ void DOS_Shell::InputCommand(char * line) {
                 break;
 
             case 0x4B00:	/* LEFT */
-                if(IS_PC98_ARCH || (isDBCSCP() && IS_DOS_JAPANESE)) {
+                if(IS_PC98_ARCH || (isDBCSCP()
+#if defined(USE_TTF)
+                    && dbcs_sbcs
+#endif
+                    && IS_DOS_JAPANESE)) {
                     if (str_index) {
-                        uint16_t count = GetWideCount(line, str_index);
+                        uint16_t count = GetLastCount(line, str_index);
                         uint8_t ch = line[str_index - 1];
                         while(count > 0) {
+                            uint16_t wide = GetWideCount(line, str_index);
                             backone();
                             str_index --;
+                            if (wide > count) str_index --;
                             count--;
                         }
                         if(CheckHat(ch)) {
@@ -572,7 +633,11 @@ void DOS_Shell::InputCommand(char * line) {
                         }
                     }
                 } else {
-                    if (isDBCSCP()&&str_index>1&&(line[str_index-1]<0||((dos.loaded_codepage==932||(dos.loaded_codepage==936&&gbk)||dos.loaded_codepage==950)&&line[str_index-1]>=0x40))&&line[str_index-2]<0) {
+                    if (isDBCSCP()
+#if defined(USE_TTF)
+                        &&dbcs_sbcs
+#endif
+                        &&str_index>1&&(line[str_index-1]<0||((dos.loaded_codepage==932||(dos.loaded_codepage==936&&gbk)||dos.loaded_codepage==950||dos.loaded_codepage==951)&&line[str_index-1]>=0x40))&&line[str_index-2]<0) {
                         backone();
                         str_index --;
                         MoveCaretBackwards();
@@ -631,11 +696,15 @@ void DOS_Shell::InputCommand(char * line) {
 				}	
         		break;
             case 0x4D00:	/* RIGHT */
-                if(IS_PC98_ARCH || (isDBCSCP() && IS_DOS_JAPANESE)) {
+                if(IS_PC98_ARCH || (isDBCSCP()
+#if defined(USE_TTF)
+                    && dbcs_sbcs
+#endif
+                    && IS_DOS_JAPANESE)) {
                     if (str_index < str_len) {
                         uint16_t count = 1;
                         if(str_index < str_len - 1) {
-                            count = GetWideCount(line, str_index + 2);
+                            count = GetLastCount(line, str_index + 2);
                         }
                         while(count > 0) {
                             outc(line[str_index++]);
@@ -643,7 +712,11 @@ void DOS_Shell::InputCommand(char * line) {
                         }
                     }
                 } else {
-                    if (isDBCSCP()&&str_index<str_len-1&&line[str_index]<0&&(line[str_index+1]<0||((dos.loaded_codepage==932||(dos.loaded_codepage==936&&gbk)||dos.loaded_codepage==950)&&line[str_index+1]>=0x40))) {
+                    if (isDBCSCP()
+#if defined(USE_TTF)
+                        &&dbcs_sbcs
+#endif
+                        &&str_index<str_len-1&&line[str_index]<0&&(line[str_index+1]<0||((dos.loaded_codepage==932||(dos.loaded_codepage==936&&gbk)||dos.loaded_codepage==950||dos.loaded_codepage==951)&&line[str_index+1]>=0x40))) {
                         outc((uint8_t)line[str_index++]);
                     }
                     if (str_index < str_len) {
@@ -735,14 +808,22 @@ void DOS_Shell::InputCommand(char * line) {
 
                 break;
             case 0x5300:/* DELETE */
-                if(IS_PC98_ARCH || (isDBCSCP() && IS_DOS_JAPANESE)) {
+                if(IS_PC98_ARCH || (isDBCSCP()
+#if defined(USE_TTF)
+                    && dbcs_sbcs
+#endif
+                    && IS_DOS_JAPANESE)) {
                     if(str_len) {
                         size += DeleteBackspace(true, line, str_index, str_len);
                     }
                 } else {
                     if(str_index>=str_len) break;
                     int k=1;
-                    if (isDBCSCP()&&str_index<str_len-1&&line[str_index]<0&&(line[str_index+1]<0||((dos.loaded_codepage==932||(dos.loaded_codepage==936&&gbk)||dos.loaded_codepage==950)&&line[str_index+1]>=0x40)))
+                    if (isDBCSCP()
+#if defined(USE_TTF)
+                        &&dbcs_sbcs
+#endif
+                        &&str_index<str_len-1&&line[str_index]<0&&(line[str_index+1]<0||((dos.loaded_codepage==932||(dos.loaded_codepage==936&&gbk)||dos.loaded_codepage==950||dos.loaded_codepage==951)&&line[str_index+1]>=0x40)))
                         k=2;
                     for (int i=0; i<k; i++) {
                         uint16_t a=str_len-str_index-1;
@@ -784,13 +865,21 @@ void DOS_Shell::InputCommand(char * line) {
                 }
                 break;
             case 0x08:				/* BackSpace */
-                if(IS_PC98_ARCH || (isDBCSCP() && IS_DOS_JAPANESE)) {
+                if(IS_PC98_ARCH || (isDBCSCP()
+#if defined(USE_TTF)
+                    && dbcs_sbcs
+#endif
+                    && IS_DOS_JAPANESE)) {
                     if(str_index) {
                         size += DeleteBackspace(false, line, str_index, str_len);
                     }
                 } else {
                     int k=1;
-                    if (isDBCSCP()&&str_index>1&&(line[str_index-1]<0||((dos.loaded_codepage==932||(dos.loaded_codepage==936&&gbk)||dos.loaded_codepage==950)&&line[str_index-1]>=0x40))&&line[str_index-2]<0)
+                    if (isDBCSCP()
+#if defined(USE_TTF)
+                        &&dbcs_sbcs
+#endif
+                        &&str_index>1&&(line[str_index-1]<0||((dos.loaded_codepage==932||(dos.loaded_codepage==936&&gbk)||dos.loaded_codepage==950||dos.loaded_codepage==951)&&line[str_index-1]>=0x40))&&line[str_index-2]<0)
                         k=2;
                     for (int i=0; i<k; i++)
                         if (str_index) {
@@ -861,7 +950,8 @@ void DOS_Shell::InputCommand(char * line) {
                 for (std::list<std::string>::iterator source = l_completion.begin(); source != l_completion.end(); ++source) {
                     std::string name = source->c_str();
                     if (col==1) {
-                        WriteOut("%s\n", name.c_str());
+                        WriteOut_NoParsing(name.c_str(), true);
+                        WriteOut("\n");
                         lastcr=true;
                         p_count++;
                     } else {
@@ -933,15 +1023,35 @@ void DOS_Shell::InputCommand(char * line) {
                 //      DOSBox / DOSBox-X have always acted as if DOSKEY is loaded in a fashion, so
                 //      we'll emulate DOSKEY behavior here.
                 while (str_index < str_len) {
+                    uint16_t count = 1, wide = 1;
+                    if(IS_PC98_ARCH || (isDBCSCP()
+#if defined(USE_TTF)
+                        && dbcs_sbcs
+#endif
+                        && IS_DOS_JAPANESE)) {
+                            count = GetLastCount(line, str_index+1);
+                            wide = GetWideCount(line, str_index+1);
+                    }
                     outc(' ');
                     str_index++;
+                    if (wide > count) str_index ++;
                 }
                 while (str_index > 0) {
+                    uint16_t count = 1, wide = 1;
+                if(IS_PC98_ARCH || (isDBCSCP()
+#if defined(USE_TTF)
+                    && dbcs_sbcs
+#endif
+                    && IS_DOS_JAPANESE)) {
+                        count = GetLastCount(line, str_index);
+                        wide = GetWideCount(line, str_index);
+                    }
                     backone();
                     outc(' ');
                     backone();
                     MoveCaretBackwards();
                     str_index--;
+                    if (wide > count) str_index --;
                 }
 
                 *line = 0;      // reset the line.
@@ -950,7 +1060,11 @@ void DOS_Shell::InputCommand(char * line) {
                 str_len = 0;
                 break;
             default:
-                if(IS_PC98_ARCH || (isDBCSCP() && IS_DOS_JAPANESE)) {
+                if(IS_PC98_ARCH || (isDBCSCP()
+#if defined(USE_TTF)
+                    && dbcs_sbcs
+#endif
+                    && IS_DOS_JAPANESE)) {
                     bool kanji_flag = false;
                     uint16_t pos = str_index;
                     while(1) {
@@ -979,7 +1093,7 @@ void DOS_Shell::InputCommand(char * line) {
                         outc(line[pos]);
                         pos++;
                     }
-                    pos = str_len;
+                    pos = GetRemoveCount(line, str_len);
                     while(pos > str_index) {
                         backone();
                         pos--;
@@ -1234,8 +1348,15 @@ continue_1:
 	p_fullname = Which(name);
 	if (!p_fullname) return false;
 	strcpy(fullname,p_fullname);
+    std::string assoc = hasAssociation(fullname);
+    if (assoc.size()) {
+        noassoc=true;
+        DoCommand((char *)(assoc+" "+fullname).c_str());
+        noassoc=false;
+        return true;
+    }
 	const char* extension = strrchr(fullname,'.');
-	
+
 	/*always disallow files without extension from being executed. */
 	/*only internal commands can be run this way and they never get in this handler */
 	if(extension == 0)
@@ -1416,9 +1537,34 @@ static const char * bat_ext=".BAT";
 static const char * com_ext=".COM";
 static const char * exe_ext=".EXE";
 static char which_ret[DOS_PATHLENGTH+4], s_ret[DOS_PATHLENGTH+4];
+extern bool wild_match(const char *haystack, char *needle);
+static std::string assocs = "";
+std::string DOS_Shell::hasAssociation(const char* name) {
+    if (noassoc) {
+        assocs = "";
+        return assocs;
+    }
+    std::string extension = strrchr(name, '.') ? strrchr(name, '.') : ".";
+    cmd_assoc_map_t::iterator iter = cmd_assoc.find(extension.c_str());
+    if (iter != cmd_assoc.end()) {
+        assocs = strcasecmp(extension.c_str(), iter->second.c_str()) ? iter->second : "";
+        return assocs;
+    }
+    std::transform(extension.begin(), extension.end(), extension.begin(), ::toupper);
+    for (cmd_assoc_map_t::iterator iter = cmd_assoc.begin(); iter != cmd_assoc.end(); ++iter) {
+        std::string ext = iter->first;
+        if (ext.find('*')==std::string::npos && ext.find('?')==std::string::npos) continue;
+        std::transform(ext.begin(), ext.end(), ext.begin(), ::toupper);
+        if (wild_match(extension.c_str(), (char *)ext.c_str())) {
+            assocs = strcasecmp(extension.c_str(), iter->second.c_str()) ? iter->second : "";
+            return assocs;
+        }
+    }
+    assocs = "";
+    return assocs;
+}
 
-bool DOS_Shell::hasExecutableExtension(const char* name)
-{
+bool DOS_Shell::hasExecutableExtension(const char* name) {
     auto extension = strrchr(name, '.');
     if (!extension) return false;
     return (!strcasecmp(extension, com_ext) || !strcasecmp(extension, exe_ext) || !strcasecmp(extension, bat_ext));
@@ -1431,9 +1577,16 @@ char * DOS_Shell::Which(char * name) {
 	/* Parse through the Path to find the correct entry */
 	/* Check if name is already ok but just misses an extension */
 
-	if (hasExecutableExtension(name)) {
+    std::string upname = name;
+    std::transform(upname.begin(), upname.end(), upname.begin(), ::toupper);
+	if (hasAssociation(name).size() && DOS_FileExists(name))
+		return name;
+	else if (hasAssociation(name).size() && DOS_FileExists(upname.c_str())) {
+		strcpy(name, upname.c_str());
+		return name;
+	} else if (hasExecutableExtension(name)) {
 		if (DOS_FileExists(name)) return name;
-		upcase(name);
+		strcpy(name, upname.c_str());
 		if (DOS_FileExists(name)) return name;
 	} else {
 		/* try to find .com .exe .bat */
@@ -1499,7 +1652,10 @@ char * DOS_Shell::Which(char * name) {
 			if((name_len + len + 1) >= DOS_PATHLENGTH) continue;
 			strcat(path,strchr(name, ' ')?("\""+std::string(name)+"\"").c_str():name);
 
-			if (hasExecutableExtension(path)) {
+			if (hasAssociation(path).size() && DOS_FileExists(path)) {
+				strcpy(which_ret,path);
+				return strchr(which_ret, '\"')&&DOS_GetSFNPath(which_ret, s_ret, false)?s_ret:which_ret;
+			} else if (hasExecutableExtension(path)) {
 				strcpy(which_ret,path);
 				if (DOS_FileExists(which_ret)) return strchr(which_ret, '\"')&&DOS_GetSFNPath(which_ret, s_ret, false)?s_ret:which_ret;
 			} else {
